/**********************************************************************************/
/*									 	  */
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved			  */	
/* cmvapi.c 									  */
/*										  */
/* This file is part of the "ADI USB ADSL Driver for Linux".			  */	
/*										  */
/* "ADI USB ADSL Driver for Linux" is free software; you can redistribute it      */
/* and/or modify it under the terms of the GNU General Public License as 	  */
/* published by the Free Software Foundation; either version 2 of the License,    */
/* or (at your option) any later version.					  */
/*										  */
/* "ADI USB ADSL Driver for Linux" is distributed in the hope that it will be     */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of	  */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		  */
/* GNU General Public License for more details.					  */
/*										  */
/* You should have received a copy of the GNU General Public License		  */
/* along with "ADI USB ADSL Driver for Linux"; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	  */
/**********************************************************************************/

#include "linuxusbapi.h"

UINT AdiReadDspVersion(DWORD *pReturnValue) 
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "info";
	structCommand.sOffSet = "10";
        structCommand.sValue = "";
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );
	
	*pReturnValue = structResponse.iValue;
	return structResponse.iReturnCode;
}


UINT AdiReadLineRates(DWORD *pReturnValue) 
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "RATE";
	structCommand.sOffSet = "0";
        structCommand.sValue = "";
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );
	
	*pReturnValue = structResponse.iValue;
	return structResponse.iReturnCode;
}


UINT AdiReadTransmitLineRate(DWORD *pLineRate)
{
	UINT iReturnCode;
	iReturnCode = AdiReadLineRates(pLineRate); 
	if ( iReturnCode != API_RETURN_SUCCESS)
		return iReturnCode;
	*pLineRate = 32 * (*pLineRate & 0xFFFF); 
	return	API_RETURN_SUCCESS;
}

UINT AdiReadReceiveLineRate(DWORD *pLineRate)
{
	UINT iReturnCode;
	iReturnCode = AdiReadLineRates(pLineRate); 
	if ( iReturnCode != API_RETURN_SUCCESS)
		return iReturnCode;
	*pLineRate = 32 * ((*pLineRate >>16) & 0xFFFF);
	return	API_RETURN_SUCCESS;
}


UINT AdiReadSTAT0(DWORD *pReturnValue) 
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "STAT";
	structCommand.sOffSet = "0";
        structCommand.sValue = "";
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );
	
	*pReturnValue = structResponse.iValue;
	return structResponse.iReturnCode;
}

UINT AdiReadSTAT1(DWORD *pReturnValue) 
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "STAT";
	structCommand.sOffSet = "1";
        structCommand.sValue = "";
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );
	
	*pReturnValue = structResponse.iValue;
	return structResponse.iReturnCode;
}

// *********************************************************************************
UINT AdiReadModemStatus(DWORD *pLinkStatus, DWORD *pAdslStandard)
{
	UINT returnCode;
	DWORD returnValue;

	returnCode = AdiReadSTAT0(&returnValue);
	*pLinkStatus = (returnValue >> 8) & 0xF; 
	*pAdslStandard = returnValue & 0x4FFF;
	return returnCode;
}

UINT	AdiReadADSLStandard(DWORD *pAdslStandard)
{
	UINT returnCode;	
	returnCode = AdiReadSTAT0(pAdslStandard);
	*pAdslStandard &= 0x4000;
	return returnCode;
}
UINT	AdiReadLinkStatus(DWORD *pLinkStatus)
{
	UINT returnCode;	
	returnCode = AdiReadSTAT0(pLinkStatus);
	*pLinkStatus = (*pLinkStatus >> 8) & 0x7; 
	return returnCode;
}

UINT AdiReadFarEndCRC(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "46";
        structCommand.sValue = "";
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}

UINT AdiReadNearEndCRC(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "22";
        structCommand.sValue = "";
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}

UINT AdiReadNearEndFEC(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "54";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT AdiReadFarEndFEC(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "58";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT AdiReadDSInterleaveDepth(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "INTL";
	structCommand.sOffSet = "0";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}

UINT AdiReadUSInterleaveDepth(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "INTL";
	structCommand.sOffSet = "1";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT AdiReadInterleavedTransmittedCells(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "3";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT AdiReadFastTransmittedCells(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "4";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}

UINT AdiReadInterleavedReceivedCells(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "11";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT AdiReadFastReceivedCells(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "12";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT AdiReadDSConnectionType(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;


	structCommand.sCMV = "CODE";
	structCommand.sOffSet = "0";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );



	if ( structResponse.iReturnCode != API_RETURN_SUCCESS )
		return structResponse.iReturnCode;

	*pReturnValue = structResponse.iValue; 
	return GetConnType(pReturnValue);
}

UINT AdiReadUSConnectionType(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "CODE";
	structCommand.sOffSet = "1";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	if ( structResponse.iReturnCode != API_RETURN_SUCCESS )
		return structResponse.iReturnCode;

	*pReturnValue = structResponse.iValue; 
	return GetConnType(pReturnValue);
}

UINT GetConnType(DWORD *CodeCmvReturn)
{

	if (( (*CodeCmvReturn & 0xFF00 ) == 0x0100 ) || 
		( (*CodeCmvReturn & 0xFF00 ) == 0x0000 )     ) 
	{
		*CodeCmvReturn = API_CONN_TYPE_INTERLEAVED;
		return API_RETURN_SUCCESS;
	}
	else
	{
		if (( ((*CodeCmvReturn>>16) & 0xFF00 ) == 0x0100 ) || 
			( ((*CodeCmvReturn>>16) & 0xFF00 ) == 0x0000  )   ) 
			{
				*CodeCmvReturn = API_CONN_TYPE_FAST;
				return API_RETURN_SUCCESS;
			}
		else
		{
			uiAPILastInternalError = INTR_CONNTYPE_EXTRACT_ERROR;
			*CodeCmvReturn = API_CONN_TYPE_NOTDETECTED;
			return API_RETURN_SUCCESS; 
		}
	}
}

UINT AdiReadFramingMode(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "OPTN";
	structCommand.sOffSet = "2";
        structCommand.sValue = "";  
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	// bits 24 & 25 indicate the Framing mode !!!
	*pReturnValue = (structResponse.iValue >> 24) & 0x3; 
	return structResponse.iReturnCode;
}



UINT AdiReadTrellisEncoding(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "OPTN";
	structCommand.sOffSet = "0";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	// 2nd bit indicates the trellis encoding ...
	*pReturnValue = (structResponse.iValue >> 1 ) & 0x1; 
	return structResponse.iReturnCode;
}

UINT AdiReadDelineation(DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "OPTN";
	structCommand.sOffSet = "2";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );
	
	*pReturnValue = API_DELINEATION_GOOD;

	if ( ( structResponse.iValue & 0x0030 ) || ( structResponse.iValue & 0x0C00 ) )
		*pReturnValue = API_DELINEATION_BAD;

	return structResponse.iReturnCode;
}

UINT	AdiReadATMCellsDropped_INTRL(DWORD *pReturnValue)
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "19";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}

UINT	AdiReadATMCellsDropped_FAST(DWORD *pReturnValue)
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "20";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT	AdiReadNearEndSNRMargin(FLOAT *pfReturnValue)
{
	// returns the NE last measured SNR margin.
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "27"; // "25"; 
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pfReturnValue = (FLOAT) (( structResponse.iValue & 0xFF)/2.0); 
	return structResponse.iReturnCode;

}


UINT	AdiReadFarEndSNRMargin(FLOAT *pfReturnValue)
{
	// returns the FE last measured SNR margin.
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "29"; //"49";
        structCommand.sValue = "";  
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pfReturnValue = (FLOAT) ((structResponse.iValue & 0xFF)/2.0); 
	return structResponse.iReturnCode;

}

// case API_CMV_NE_ATTENUATION: // DIAG 23 (FLOAT)
UINT	AdiReadNearEndAttenuation(FLOAT *pfReturnValue)
{
	// returns the NE last measured attenuation.
	// reported in 1/2db steps. (15.1)
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "23";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pfReturnValue = (FLOAT)((structResponse.iValue & 0xFF) / 2.0); 
	return structResponse.iReturnCode;

}

// 	case API_CMV_FE_ATTENUATION: // DIAG 47	(FLOAT)
UINT	AdiReadFarEndAttenuation(FLOAT *pfReturnValue)
{

	// returns the FE last measured attenuation.
	// reported in 1/2db steps. (15.1)

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "DIAG";
	structCommand.sOffSet = "47";
        structCommand.sValue = ""; 
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pfReturnValue = (FLOAT)((structResponse.iValue & 0xFF) / 2.0); 
	return structResponse.iReturnCode;

}

UINT	AdiReadRTVendorID(DWORD *pReturnValue)
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "INFO";
	structCommand.sOffSet = "13";
        structCommand.sValue = "";  

	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;

}

UINT	AdiReadCOVendorID(DWORD *pReturnValue)		
{
	// INFO 14, INFO 20 (see the doc.) ??? 20 looks wrong !
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "INFO";
	structCommand.sOffSet = "14";
        structCommand.sValue = ""; 

	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT	AdiReadLineType(DWORD *pReturnValue)
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = "OPTN";
	structCommand.sOffSet = "4";
	structCommand.sValue = "";

	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue; 
	return structResponse.iReturnCode;
}


UINT	AdiUSBinLoading(UINT uiBin, DWORD *pNumBitPerFrame)
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;
        char  tempstr[10];

	if ( ( uiBin < API_US_BIN0_OFFSET ) || ( uiBin > API_US_BIN30_OFFSET) )
		return API_RETURN_FUNC_INPUT_ERROR;

	structCommand.sCMV = "TONE";
        sprintf(tempstr,"%d",uiBin);
	structCommand.sOffSet = tempstr;
        structCommand.sValue = ""; 

	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pNumBitPerFrame = structResponse.iValue & 0xF; 
	return structResponse.iReturnCode;
}

UINT	AdiDSBinLoading(UINT uiBin, DWORD *pNumBitPerFrame, FLOAT *pSNR, FLOAT *pGain)
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;
	DWORD dwTemp;
        UCHAR tempstr[10];
	

	if ( ( uiBin < API_DS_BIN31_OFFSET ) || ( uiBin > API_DS_BIN255_OFFSET) )
		return API_RETURN_FUNC_INPUT_ERROR;

	structCommand.sCMV = "TONE";
        sprintf(tempstr,"%d",uiBin);
	structCommand.sOffSet = tempstr;
        structCommand.sValue = ""; 

	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pNumBitPerFrame = structResponse.iValue & 0xF;
	dwTemp = (structResponse.iValue >> 16) & 0xFFFF;
	*pSNR = (FLOAT)(((dwTemp >> 8 ) & 0xFF )+ (dwTemp & 0xFF) / 256.0);  // 8.8
	dwTemp = (structResponse.iValue >> 4) & 0x00000FFF;
	*pGain = (FLOAT)(((dwTemp >> 9 ) & 0x7 )+ (dwTemp & 0x1FF) / 512.0); // 3.9
	return structResponse.iReturnCode;

}


UINT	AdiModemSelfTest(DWORD *pTestResult)
{
	
	 //For the self test we have to reset the modem and then call STAT0;
	 //Then the first four bit should be zero to pass the self test !
	 
	UINT iReturnCode;
	DWORD dwSTAT0;
	iReturnCode = AdiModemHardReset();
	if ( iReturnCode == -1 )
		return API_RETURN_COMM_ERROR;
        sleep(4);
	iReturnCode = AdiReadSTAT0(&dwSTAT0);
	if ( iReturnCode != API_RETURN_SUCCESS )
		return iReturnCode;
	
	dwSTAT0 &= 0xF;
	if ( dwSTAT0 == 0x0 )
	{
		*pTestResult = API_MODEM_TEST_PASS;
	}
	else
	{
		*pTestResult = API_MODEM_TEST_FAILURE;
	}	
	return 	API_RETURN_SUCCESS;;
}


UINT	AdiAPIFunctionRead(int funcCode, PAPI_COMM_BUFFER pCommBuffer)
{

	switch	(funcCode)
	{

	case API_CMV_DSP_VERSION:
		return AdiReadDspVersion(&pCommBuffer->dwValue);  
		break;

	case API_CMV_LINE_RATE:
		return AdiReadLineRates(&pCommBuffer->dwValue); 
		break;

	case API_CMV_TRANSMIT_LINERATE:
		return AdiReadTransmitLineRate(&pCommBuffer->dwValue); 	
		break;

	case API_CMV_RECEIVE_LINERATE:
		return AdiReadReceiveLineRate(&pCommBuffer->dwValue); 
		break;

	case API_CMV_STATUS:
		return AdiReadSTAT0(&pCommBuffer->dwValue); 
		break;

	case API_CMV_MODEM_STATUS:
		return AdiReadLinkStatus(&pCommBuffer->dwValue);
		break;

	case API_CMV_ADSL_STANDARD_READ:
		return AdiReadADSLStandard(&pCommBuffer->dwValue); 
		break;

	case API_CMV_DS_INTERLEAVE_DEPTH:	// INTL 0

		return AdiReadDSInterleaveDepth(&pCommBuffer->dwValue); 
		break;

	case API_CMV_US_INTERLEAVE_DEPTH:	// INTL 1
		return AdiReadUSInterleaveDepth(&pCommBuffer->dwValue); 
		break;

	case API_CMV_INTL_TRANS_CELLS:	// DIAG 3

		return AdiReadInterleavedTransmittedCells(&pCommBuffer->dwValue); 
		break;

	case API_CMV_FAST_TRANS_CELLS:	// DIAG 4

		return AdiReadFastTransmittedCells(&pCommBuffer->dwValue); 
		break;

	case API_CMV_INTL_RECEIVED_CELLS:	// DIAG 11

		return AdiReadInterleavedReceivedCells(&pCommBuffer->dwValue);
		break;

	case API_CMV_FAST_RECEIVED_CELLS:	// DIAG 12

		return AdiReadFastReceivedCells(&pCommBuffer->dwValue);
		break;

	case API_CMV_CRC_NE:	// DIAG 22

		return AdiReadNearEndCRC(&pCommBuffer->dwValue);
		break;

	case API_CMV_CRC_FE	:	// DIAG 46

		return AdiReadFarEndCRC(&pCommBuffer->dwValue);
		break;

	case API_CMV_FEC_NE	:	// DIAG 54

		return AdiReadNearEndFEC(&pCommBuffer->dwValue);
		break;

	case API_CMV_FEC_FE	:	// DIAG 58

		return AdiReadFarEndFEC(&pCommBuffer->dwValue);
		break;

	case API_CMV_NE_SNR_MARGIN:	// DIAG 25 (FLOAT)
		return AdiReadNearEndSNRMargin(&pCommBuffer->fValue);
		break;

	case API_CMV_FE_SNR_MARGIN: // DIAG 49 (FLOAT)
		return AdiReadFarEndSNRMargin(&pCommBuffer->fValue);
		break;

	case API_CMV_NE_ATTENUATION: // DIAG 23 (FLOAT)
		return AdiReadNearEndAttenuation(&pCommBuffer->fValue);
		break;

	case API_CMV_FE_ATTENUATION: // DIAG 47	(FLOAT)
		return AdiReadFarEndAttenuation(&pCommBuffer->fValue);
		break;

	case API_CMV_DELINEATION_LATCHED:	// DIAG 10 ( USING DIAG 2)
		return AdiReadDelineation(&pCommBuffer->dwValue);
		break;

	case API_CMV_ATMCELL_DROPPED_INTRL:	// DIAG 19
		return AdiReadATMCellsDropped_INTRL(&pCommBuffer->dwValue);
		break;

	case API_CMV_ATMCELL_DROPPED_FAST:	//DIAG 20
		return AdiReadATMCellsDropped_FAST(&pCommBuffer->dwValue);
		break;

	case API_CMV_DS_CONN_TYPE:		// CODE 0
		return AdiReadDSConnectionType(&pCommBuffer->dwValue);
		break;

	case API_CMV_US_CONN_TYPE:		// CODE 1
		return AdiReadUSConnectionType(&pCommBuffer->dwValue);
		break;

	case API_CMV_FRAME_OVERHEAD:	// OPTN 2
		return AdiReadFramingMode(&pCommBuffer->dwValue);
		break;

	case API_CMV_TRELLIS_ENCODING:	// OPTN 0
		return AdiReadTrellisEncoding(&pCommBuffer->dwValue);
		break;
		
	case API_CMV_CO_VENDOR_ID:	// INFO 14
		return AdiReadCOVendorID(&pCommBuffer->dwValue);
		break;
	case API_CMV_RT_VENDOR_ID:  // INFO 13
		return AdiReadRTVendorID(&pCommBuffer->dwValue);
		break;
	case API_MODEM_TEST:	// modem self-test
		return AdiModemSelfTest(&pCommBuffer->dwValue); 
		break;

	default:
		return API_RETURN_FUNC_NOTSUPPORTED;
		break;
	}

	return	API_RETURN_FUNC_NOTSUPPORTED;
}

UINT WaitForModemBoot(void)
{
  int State,ret,cnt=0;
  while (1)
  {
    ret = GetModemState(&State);
    if (ret!=API_RETURN_SUCCESS)
    {
      return ret;
    }
    switch (State)
    {
      case STATE_UNDEFINED:
      case STATE_JUST_PLUGGED_IN:
      case STATE_UNTRAIN:
      case STATE_UNTRAIN_TX:
      case STATE_UNTRAIN_RX:
      case STATE_HARD_RESET_INITIATE:
      case STATE_HARD_RESET_END:
      case STATE_BOOT_WAIT:
        sleep(1);
        cnt++;
        if (cnt>12)
        { 
          return API_RETURN_HARDWARE_ERROR;
        } 
        break;
      default:
        return API_RETURN_SUCCESS;
    }
   }
   printf("Error3:%d\n",State);
   return API_RETURN_SUCCESS;
}

UINT AdiModemTryHardReset(void)
{
   UINT ret;
   PVOID buf="";
   ret =  AdiIOCTLCommand(ADIUSBADSLRESET,buf);
   ret = WaitForModemBoot();
   return ret;
}

UINT AdiModemHardReset(void)
{
  int i,ret;
  for (i=0;i<3;i++)
  {
    ret = AdiModemTryHardReset();
    if (ret==API_RETURN_SUCCESS)
      return ret;
  }
  return ret;
}

UINT AdiModemSoftReset(void)
{
   UINT ret;
   PVOID buf="";
   ret =  AdiIOCTLCommand(ADIUSBADSLSOFTRST,buf);
   ret = WaitForModemBoot();
   return ret;
}


UINT AdiRestartDSLConnection(void)
{
  UINT ret;
	//CMV_COMMAND structCommand;
	//ADICMV_RESPONSE structResponse;

	//structCommand.sCMV = "CNTL";
	//structCommand.sOffSet = "0";
	//structCommand.sValue = "0";
	//CMVReadWrite(CMV_WRITE_COMMAND,structCommand , &structResponse );
        ret = AdiModemSoftReset();
        return ret;
}



UINT AdiModemTestRequest(void)
{
   UINT ret;
   PVOID buf="";
   ret =  AdiIOCTLCommand(ADIUSBADSLTESTREQ,buf);
   ret = WaitForModemBoot();
   if (ret!=API_RETURN_SUCCESS)
     ret = WaitForModemBoot();
   return ret;
}


UINT	AdiWriteAdslStandard(DWORD dwAdslStandard)
{
	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;
        char tempstr[10];

	structCommand.sCMV = "OPTN";
	structCommand.sOffSet = "4";
        sprintf(tempstr,"%lx",dwAdslStandard);
	structCommand.sValue = tempstr;

	CMVReadWrite(CMV_WRITE_COMMAND,structCommand , &structResponse );

	return structResponse.iReturnCode;
}


UINT	AdiSetModemTestSignalState(void)
{
	DWORD  Stat0;


        AdiModemTestRequest();
        sleep(4);
        AdiReadSTAT0(&Stat0);
        Stat0 &= 0x700;
        if (Stat0==0x400) return API_RETURN_SUCCESS;
        else return API_RETURN_HARDWARE_ERROR;
}


UINT	AdiSetModemNormalState(void)
{
	UINT ret;
	
        ret = AdiModemHardReset();
        return ret;
}


UINT	AdiControlBinTest(const BINTEST_COMMAND	*binTestCmd)
{
  char tempstr[10];

	DWORD	dwWriteValue = 0x0;

	ULONG transmitSignalType = binTestCmd->transmitSignalType;
	ULONG binNumber = binTestCmd->binNumber;
	ULONG tonePower = binTestCmd->tonePower;
	ULONG widebandPower = binTestCmd->widebandPower;

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	dwWriteValue |= (transmitSignalType & 0xFF);
	dwWriteValue |= ((binNumber << 8) & 0xFF00);
	dwWriteValue |= ((tonePower << 16) & 0xFF0000);
	dwWriteValue |= ((widebandPower << 24) & 0xFF000000);
	
	structCommand.sCMV = "TEST";
	structCommand.sOffSet = "0";
        sprintf(tempstr,"%lx",dwWriteValue);
	structCommand.sValue = tempstr;
        //printf("tempstr: %s\n",tempstr);
	CMVReadWrite(CMV_WRITE_COMMAND,structCommand , &structResponse );

	return structResponse.iReturnCode;
}

UINT	AdiReadBinPower(UINT binNum, DWORD *pReturnValue)
{

	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;
        char  tempstr[10];

	structCommand.sCMV = "TEST";
        sprintf(tempstr,"%d",binNum);
	structCommand.sOffSet = tempstr;
        structCommand.sValue  = "";

	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );

	*pReturnValue = structResponse.iValue & 0xFF; 
	return structResponse.iReturnCode;

}


UINT	AdiAPIFunctionWrite(int funcCode, const API_COMM_BUFFER *buf)
{

	switch	(funcCode)
	{

	case API_CMV_ADSL_STANDARD_WRITE:
		return AdiWriteAdslStandard(buf->dwValue);

	case API_CMV_RESTART_DSL:	// CTRL 0 0 ( it is soft reset!!!);
		return AdiRestartDSLConnection();
		break;

	default:
		return API_RETURN_FUNC_NOTSUPPORTED;
		break;
	}

	return	API_RETURN_FUNC_NOTSUPPORTED;
}


UINT	AdiAPICmvRead(UCHAR *Comm,UCHAR *Offset,DWORD *pReturnValue)
{
 	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = Comm;
	structCommand.sOffSet = Offset;
        structCommand.sValue = "";
	CMVReadWrite(CMV_READ_COMMAND,structCommand , &structResponse );
	
	*pReturnValue = structResponse.iValue;
	return structResponse.iReturnCode;
}

UINT	AdiAPICmvWrite(UCHAR *Comm,UCHAR *Offset, UCHAR *Value,DWORD *pReturnValue)
{
 	CMV_COMMAND structCommand;
	ADICMV_RESPONSE structResponse;

	structCommand.sCMV = Comm;
	structCommand.sOffSet = Offset;
        structCommand.sValue = Value;
	CMVReadWrite(CMV_WRITE_COMMAND,structCommand , &structResponse );
	
	*pReturnValue = structResponse.iValue;
	return structResponse.iReturnCode;
}


















