/**********************************************************************************/
/*										  */
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved			  */	
/* ioctl.c 									  */
/*										  */
/* This file is part of the "ADI USB ADSL Driver for Linux".			  */	
/*										  */
/* "ADI USB ADSL Driver for Linux" is free software; you can redistribute it      */
/* and/or modify it under the terms of the GNU General Public License as 	  */
/* published by the Free Software Foundation; either version 2 of the License,    */
/* or (at your option) any later version.					  */
/*										  */
/* "ADI USB ADSL Driver for Linux" is distributed in the hope that it will be     */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of	  */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		  */
/* GNU General Public License for more details.					  */
/*										  */
/* You should have received a copy of the GNU General Public License		  */
/* along with "ADI USB ADSL Driver for Linux"; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	  */
/**********************************************************************************/

#include "linuxusbapi.h"

HANDLE OpenDriver( void ) 
{
    char rootname[PATH_MAX] = "";
    char busname[PATH_MAX] = "";
    char devname[PATH_MAX] = "";
    DIR *rootdir;
    DIR *busdir;
    struct dirent *rootentry;
    struct dirent *busentry;
    usb_device_descriptor descriptor;
    int bMatch = FALSE;
    int fd, ret, drv;
 
    sprintf(rootname, "/proc/bus/usb");

    /* open rootdir and iterate on contents */
    rootdir = opendir(rootname);

    while ((rootentry = readdir(rootdir)) != NULL) {
        /* ignore . and .. entries although it would work anyway */
        if (rootentry->d_name[0] == '.') {
            continue;
        }

        /* process each entry which is also a directory */
        sprintf(busname, "%s/%s", rootname, rootentry->d_name);

        busdir = opendir(busname);
        if (busdir == NULL) {
            continue;
        } 

        while ((busentry = readdir(busdir)) != NULL) {
            /* This dir is a bus, and each entry a device */
            /* Open each device and check the descriptor */
            /* ignore . and .. entries although it would work anyway */
            if (busentry->d_name[0] == '.') {
                continue;
            }
            sprintf(devname, "%s/%s", busname, busentry->d_name);

            fd = open(devname, O_RDONLY);
            if (fd < 0) {
                continue;
            }

            ret = read(fd, &descriptor, sizeof(usb_device_descriptor));
            close(fd);

            if (ret < 0) {
                continue;
            }


          if ((descriptor.idVendor == kEAGLE_VID) &&
              ((descriptor.idProduct==kEAGLEI_PID_PSTFIRMWARE)||(descriptor.idProduct == kEAGLEII_PID_PSTFIRMWARE)||
               (descriptor.idProduct == kEAGLEIIC_PID_PSTFIRMWARE)))
          {
             /* terminate processing, we're done */
             bMatch = TRUE;
             break;
          }            
        }
        closedir(busdir);

        if (bMatch) {
            break;
        }
    }
    closedir(rootdir);

    // see if we broke or blew through the end
    if (!bMatch) 
    {
        //ZAP(DEBUG_INIT, DEBUG_ERRORS, ("Failed to find driver\n"));
	return -1;
    }

    drv = open(devname, O_RDWR);
    if (drv < 0)
    {
        //ZAP(DEBUG_INIT, DEBUG_ERRORS, ("Failed to open driver\n"));
	return -2;
    }

    //ZAP(DEBUG_INIT, DEBUG_ATTN, ("File handle=%x\n", drv));
    return drv;
}



HANDLE DetectDriver( void ) 
{
    char rootname[PATH_MAX] = "";
    char busname[PATH_MAX] = "";
    char devname[PATH_MAX] = "";
    DIR *rootdir;
    DIR *busdir;
    struct dirent *rootentry;
    struct dirent *busentry;
    usb_device_descriptor descriptor;
    int bMatch = FALSE;
    int fd, ret;
 
    sprintf(rootname, "/proc/bus/usb");

    /* open rootdir and iterate on contents */
    rootdir = opendir(rootname);

    while ((rootentry = readdir(rootdir)) != NULL) {
        /* ignore . and .. entries although it would work anyway */
        if (rootentry->d_name[0] == '.') {
            continue;
        }

        /* process each entry which is also a directory */
        sprintf(busname, "%s/%s", rootname, rootentry->d_name);

        busdir = opendir(busname);
        if (busdir == NULL) {
            continue;
        } 

        while ((busentry = readdir(busdir)) != NULL) {
            /* This dir is a bus, and each entry a device */
            /* Open each device and check the descriptor */
            /* ignore . and .. entries although it would work anyway */
            if (busentry->d_name[0] == '.') {
                continue;
            }
            sprintf(devname, "%s/%s", busname, busentry->d_name);

            fd = open(devname, O_RDONLY);
            if (fd < 0) {
                continue;
            }

            ret = read(fd, &descriptor, sizeof(usb_device_descriptor));
            close(fd);

            if (ret < 0) {
                continue;
            }


          if ((descriptor.idVendor == kEAGLE_VID) &&
              ((descriptor.idProduct==kEAGLEI_PID_PSTFIRMWARE)||(descriptor.idProduct == kEAGLEII_PID_PSTFIRMWARE)||
               (descriptor.idProduct == kEAGLEIIC_PID_PSTFIRMWARE)))
          {
             /* terminate processing, we're done */
             bMatch = TRUE;
             break;
          }            
        }
        closedir(busdir);

        if (bMatch) {
            break;
        }
    }
    closedir(rootdir);

    // see if we broke or blew through the end
    if (!bMatch) 
    {
        //ZAP(DEBUG_INIT, DEBUG_ERRORS, ("Failed to find driver\n"));
	return -1;
    }
    return 1;
}




int AdiIOCTLCommand (
        DWORD   code,
	PVOID pBuffer
)
{
  USHORT bResult;
  HANDLE driverHandle;
  struct usbdevfs_ioctl ioctl_wrapper;
  uid_t  uid,ret;

  
  uid = geteuid();
  ret = seteuid(0);
  if (ret<0)
    return -1;

  driverHandle = OpenDriver();

  if (driverHandle <0 )
  {
    seteuid(uid);
    return -1;
  }

  ioctl_wrapper.ifno = 1;
  ioctl_wrapper.ioctl_code = code;
  ioctl_wrapper.data = pBuffer; //NULL;
  bResult = ioctl(driverHandle, USBDEVFS_IOCTL, &ioctl_wrapper);
	
  close(driverHandle);

  seteuid(uid);

  if (bResult == 0)
    return -1;

  return bResult;
}


void strupper(UCHAR *s)
{
  int l,i;
  l = strlen(s);
  for (i=0;i<l;i++)
    s[i] = toupper(s[i]);
}

void CMVReadWrite
(
	CString sReadWrite,
	CMV_COMMAND GUICommand, 
	ADICMV_RESPONSE *pDriverResponse
)
{

	unsigned int bytesRead;
	int cntr = 0;
	mp_message_t    tmp_msg;
        UCHAR           tempstr[MAX_TEMP_STR];
	API_LINK_INFO AdicmvLink;

	memset(&AdicmvLink,' ',sizeof(API_LINK_INFO));


	AdicmvLink.ReturnStatus = 1;  // some dummy number ....

	tmp_msg.preamble =		0x535c;
	tmp_msg.type =			MP_FUNCTION_TYPE_MEMACCESS;
	tmp_msg.sender_id =		SUBTYPE_ID_ME;
	tmp_msg.receiver_id =	SUBTYPE_ID_ADSL;
	tmp_msg.index =			0xDCDC;
	tmp_msg.offset_addr =	0x0; 	
	tmp_msg.data =			0x0;  

	
	if ( sReadWrite == CMV_WRITE_COMMAND )
	{
		tmp_msg.sub_type = SUBTYPE_MEMACCESS_REQWRITE;
	} 
	else if ( sReadWrite == CMV_READ_COMMAND )
	{
		tmp_msg.sub_type = SUBTYPE_MEMACCESS_REQREAD;
	}
	else
	  return;

	// converting the cmv command to upper case

        strcpy(tmp_msg.symb_addr,GUICommand.sCMV); 
        strupper(tmp_msg.symb_addr);

	// checking and assigning the offset
        strcpy(tempstr,GUICommand.sOffSet);

	for ( cntr = 0; cntr < strlen(GUICommand.sOffSet); cntr++)
	{
	  if ( isxdigit(tempstr[cntr]) == 0 )
	  {
	     strcpy(pDriverResponse->sMessage , " Error: incorrect Offset!!");
	     pDriverResponse->iReturnCode = API_RETURN_COMM_ERROR;
	     uiAPILastInternalError = INTR_CMVOFFSET_ERROR;
	     return;
	  }
	}

	sscanf(tempstr,"%hu",&tmp_msg.offset_addr);

	// checking and assigning the value
        strcpy(tempstr,GUICommand.sValue);
	for ( cntr = 0; cntr < strlen(GUICommand.sValue); cntr++)
	{
  	  if ( isxdigit(tempstr[cntr]) == 0 )
	  {
	     strcpy(pDriverResponse->sMessage , " Error: incorrect value!!");
	     pDriverResponse->iReturnCode = API_RETURN_COMM_ERROR;
	     uiAPILastInternalError = INTR_CMVVALUE_ERROR;

	     return;
	  }
	}
	sscanf(tempstr,"%lX",&tmp_msg.data);

 
	// encode the command 
	if (mp_msg_encode(&tmp_msg,(unsigned short *)&AdicmvLink.DataBuffer) != MP_ENCODE_OK )
	{

	  strcpy(pDriverResponse->sMessage , " *** ERROR : CMV ENCODE Failure. \n");;
	  pDriverResponse->iReturnCode = API_RETURN_COMM_ERROR;
	  uiAPILastInternalError = INTR_CMVENCODE_ERROR;

	  return;
	}
	else
	{
          AdicmvLink.ReturnStatus = API_RETURN_COMM_ERROR;
          bytesRead = AdiIOCTLCommand(ADIUSBADSLCMV,&AdicmvLink);
 	  if ( bytesRead == -1 )
	  {
	     strcpy(pDriverResponse->sMessage,"IOCTL ERROR!!");
	     pDriverResponse->iReturnCode = API_RETURN_COMM_ERROR;
	     // defalut the return value in case of error
	     pDriverResponse->iValue = API_DEFAULT_RETURN_VALUE; 
 	     uiAPILastInternalError = INTR_DRIVER_ERROR;
	     return;
	  }
	  if( AdicmvLink.ReturnStatus != 0 )
 	  {
	    pDriverResponse->iValue = API_DEFAULT_RETURN_VALUE; 
	    if ( AdicmvLink.ReturnStatus == 1 )
	    {
	       pDriverResponse->iReturnCode = API_RETURN_MODEMBOOTING_ERROR;
	       uiAPILastInternalError = INTR_CMV_PENDING_ERROR;
	       sprintf(pDriverResponse->sMessage,"User CMV Is not allowed now! - Status <0x%.8x>\n",AdicmvLink.ReturnStatus);
    	       return;
    	    }
	    if ( AdicmvLink.ReturnStatus == 2 )
	    {
	       pDriverResponse->iReturnCode = API_RETURN_COMM_ERROR;
	       uiAPILastInternalError = API_RETURN_COMM_ERROR;
	       sprintf(pDriverResponse->sMessage,"Internal data is not initialized! - Status <0x%.8x>\n",AdicmvLink.ReturnStatus);
    	       return;
    	    }
	    sprintf(pDriverResponse->sMessage,"CMV send failure - Status <0x%.8x>\n",AdicmvLink.ReturnStatus);
  	    // defalut the return value in case of error
            pDriverResponse->iReturnCode = API_RETURN_HARDWARE_ERROR;
	    uiAPILastInternalError = INTR_CMVSEND_ERROR;
	    return;
	  }
		
 	  // decodes the result returned from the driver ...
 	  if( mp_msg_decode(&tmp_msg,(unsigned short *)&AdicmvLink.DataBuffer) == MP_ENCODE_OK )

	  {
	    sprintf(pDriverResponse->sValue,"0x%.8lx", tmp_msg.data);
	    //pDriverResponse->iValue = tmp_msg.data;
            pDriverResponse->iValue = AdicmvLink.DataWord;
	    strcpy(pDriverResponse->sMessage , " CMV SUCCESS ") ;
	    pDriverResponse->iReturnCode = API_RETURN_SUCCESS;
	    uiAPILastInternalError = INTR_SUCCESS;
	  }
	  else
	  {
	     // defalut the return value in case of error
	     pDriverResponse->iValue = API_DEFAULT_RETURN_VALUE; 
	     strcpy(pDriverResponse->sMessage , "Error: Could not decode ...\n");
	     pDriverResponse->iReturnCode = API_RETURN_CMVDECODE_ERROR;
	     uiAPILastInternalError = INTR_CMVDECODE_ERROR;
	  }
        }

	return;
}





















