/**********************************************************************************/
/*										  */
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved			  */	
/* main.c 									  */
/*										  */
/* This file is part of the "ADI USB ADSL Driver for Linux".			  */	
/*										  */
/* "ADI USB ADSL Driver for Linux" is free software; you can redistribute it      */
/* and/or modify it under the terms of the GNU General Public License as 	  */
/* published by the Free Software Foundation; either version 2 of the License,    */
/* or (at your option) any later version.					  */
/*										  */
/* "ADI USB ADSL Driver for Linux" is distributed in the hope that it will be     */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of	  */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		  */
/* GNU General Public License for more details.					  */
/*										  */
/* You should have received a copy of the GNU General Public License		  */
/* along with "ADI USB ADSL Driver for Linux"; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	  */
/**********************************************************************************/


#include "linuxusbapi.h"
char HexValue[10];
char *LinkStatusString[]={"R-ACT-REQ(or R-TONE-REQ)"
                         ,"INITIALIZATION"
                         ,"OPERATIONAL"
                         ,"FAIL"
                         ,"TEST-SIGNAL"
                         ,"TEST-HW"
                         ,"TEST-LOOPBACK"
                         ,"FAST-RETRAIN"};
char *EncapsulationString[] = {"MPOA MODE BRIDGED ETHERNET LLC"
 			      ,"MPOA MODE BRIDGED ETHERNET VC"
 			      ,"MPOA MODE ROUTED IP LLC"
 			      ,"MPOA MODE ROUTED IP VC"
 			      ,"MPOA MODE PPPOA LLC"
 			      ,"MPOA MODE PPPOA VC"};
char *Hex(DWORD val)
{
  sprintf(HexValue,"%lx",val);
  strupper(HexValue);
  return HexValue;
}

void ClearScr(void)
{
  system("clear");
  return;
}

void ShowError(int err)
{
  char *sError;
  switch (err)
  {
    case API_RETURN_FUNC_NOTSUPPORTED:
      sError = "Function Not Supported!\n";
      break;
    case API_RETURN_HARDWARE_ERROR:
      sError = "Hardware doesn't respond!\n";
      break;
    case API_RETURN_COMM_ERROR:
      sError = "Unsuccessfull IOCTL!\n";
      break;
    case API_RETURN_FUNC_INPUT_ERROR:
      sError = "Input Error\n";
      break;
    case API_RETURN_MODEMBOOTING_ERROR:
      sError = "Modem is booting, User CMV is not allowed right now!\n";
      break;
    case API_RETURN_CMVDECODE_ERROR:
      sError = "Error in decoding reply packet!\n";
      break;
    default:
      sError = "Unknown error!\n";
      break; 
  }
  printf (" Error: %s \n",sError);
} 

void  ApiDetect(void)
{
 int ret;
 char ch;

 ret = AdiDetectDevice();
 if (ret)
   printf("\n Usb Modem Detected\n");
 else
   printf("\n Usb Modem Not Detected\n");

 printf("\n\n----------------------- Press Enter ----------------------------\n");
 do
  ch=getchar(); 
 while (ch!=10);
}

void  ApiMac(void)
{
 int ret;
 UCHAR  Mac[8];
 UCHAR  sMac[17]; 
 char   ch;

 ret = AdiAPIMacAddress(Mac,sMac);
 strupper(sMac);
 if (ret == API_RETURN_SUCCESS)
   printf(" Mac Address: %c%c:%c%c:%c%c:%c%c:%c%c:%c%c\n",sMac[0],sMac[1],sMac[2],sMac[3],sMac[4],sMac[5],
   sMac[6],sMac[7],sMac[8],sMac[9],sMac[10],sMac[11]);
 else
   ShowError(ret);

 printf("\n\n----------------------- Press Enter ----------------------------\n");
 do
  ch=getchar(); 
 while (ch!=10);
}

void  ApiVersions(void)
{
  UCHAR  *sVer;
  char ch=0;

  do {
  ClearScr();
  printf(" 1- Driver Version\n");
  printf(" 2- API Libary Version\n");
  printf(" 3- Firmware Version\n");
  printf(" 4- DSP Version\n");
  //printf(" 5- API Libary Release Date\n");
  printf(" 5- Return to previous menu\n");
  printf(" Please enter your choice>");
  ch = getchar();
  getchar();

  switch (ch)
  {
     case '1':
         sVer = AdiAPIGetVersion(API_DRIVER_VERSION);
         printf(" Driver Version -> %s\n",sVer);
         break;
     case '2':
         sVer = AdiAPIGetVersion(API_LIB_RELEASE_VERSION);
         printf(" API Library Version -> %s\n",sVer);
         break;
     case '3': 
         sVer = AdiAPIGetVersion(API_FIRMWARE_VERSION);
         strupper(sVer);
         printf(" Firmware Version -> %s\n",sVer);
         break;  
     case '4': 
         sVer = AdiAPIGetVersion(API_CMV_DSP_VERSION);
         strupper(sVer);
         printf(" DSP Version -> %s\n",sVer);
         break;  
     //case '5':
     //   sVer =  AdiAPILibReleaseDate();
     //   printf(" API Release Date -> %s\n",sVer);
     //   break;
     case '5':
        return;
  }
   printf("\n\n----------------------- Press Enter ----------------------------\n");
   do
    ch=getchar(); 
   while (ch!=10);
  }
  while (ch!='6');
}

void ApiRead(void)
{
  UINT ret,in;
  API_COMM_BUFFER  CommBuffer;
  PAPI_COMM_BUFFER pCommBuffer=&CommBuffer;
  BOOLEAN f=FALSE,ResultShowed=FALSE;   
  char ch;
  UINT uiBin;
  DWORD NumBitPerFrame;
  FLOAT SNR,Gain;

  do {
  ClearScr();
  ResultShowed=FALSE;
  printf(" 1-  Read DSP Version                  \t");
  printf(" 2-  Read Line Rate Register           \n");
  printf(" 3-  Read Transmit Line Rate           \t");
  printf(" 4-  Read Receive Line Rate            \n");
  printf(" 5-  Read Status Register              \t");
  printf(" 6-  Read Link Status                  \n");
  printf(" 7-  Read ADSL Standard                \t");
  printf(" 8-  Read Downstream interleave depth  \n");
  printf(" 9-  Read Upstream interleave depth    \t");
  printf(" 10- Read Interleaved Transmitted Cells\n");
  printf(" 11- Read Fast Transmitted Cells       \t"); 
  printf(" 12- Read Interleaved Received Cells   \n");
  printf(" 13- Read Fast Received Cells          \t");
  printf(" 14- Read Near End CRC                 \n");
  printf(" 15- Read Far End CRC                  \t");
  printf(" 16- Read Near End FEC                 \n");
  printf(" 17- Read Far End FEC                  \t");
  printf(" 18- Read Near End SNR Margin          \n");
  printf(" 19- Read Far End SNR Margin           \t");
  printf(" 20- Read Far End Attenuation          \n");
  printf(" 21- Read Delineation                  \t");
  printf(" 22- Read ATM Cells Dropped INTRL      \n");
  printf(" 23- Read ATM Cells Dropped FAST       \t");
  printf(" 24- Read Downstream Connection Type   \n");
  printf(" 25- Read Upstream Connection Type     \t");
  printf(" 26- Read Framing Mode                 \n");
  printf(" 27- Read Trellis Encoding             \t");
  printf(" 28- Read CO Vendor ID                 \n");
  printf(" 29- Read RT Vendor ID                 \t");
  printf(" 30- Read Upstream Bin Loading         \n");
  printf(" 31- Read DownStream Bin Loading       \t");
  printf(" 32- Read VPI				 \n");
  printf(" 33- Read VCI                          \t");
  printf(" 34- Read Encapsulation Type           \n");
  printf(" 35- Read Driver Type                  \t");
  printf(" 36- Read Line Type                    \n");
  printf(" 40- Return to previous menu           \n");
  printf(" Please enter your choice>");
  scanf("%d",&in);
  getchar();

  f=FALSE;
  ret=-1;
  switch (in)
  {
    case 1:
       ret = AdiReadDspVersion(&pCommBuffer->dwValue);  
       break;
    case 2:
       ret = AdiReadLineRates(&pCommBuffer->dwValue);
       break;
    case 3:
       ret = AdiReadTransmitLineRate(&pCommBuffer->dwValue); 	
       printf(" Value -> %ld bps\n",pCommBuffer->dwValue);
       ResultShowed=TRUE;
       break;
    case 4:
       ret = AdiReadReceiveLineRate(&pCommBuffer->dwValue); 
       printf(" Value -> %ld bps\n",pCommBuffer->dwValue);
       ResultShowed=TRUE;
       break;
    case 5:
       ret = AdiReadSTAT0(&pCommBuffer->dwValue); 
       break;
    case 6:
       ret = AdiReadLinkStatus(&pCommBuffer->dwValue);
       printf(" Status -> %s\n",LinkStatusString[pCommBuffer->dwValue]);
       ResultShowed=TRUE;
       break;
    case 7:
       ret = AdiReadADSLStandard(&pCommBuffer->dwValue);
       switch (pCommBuffer->dwValue)
       {
          case 0:
             printf(" Standard: Full Rate\n");
             break;
          case 1:
             printf(" Standard: Lite\n");
             break;
       } 
       ResultShowed=TRUE; 
       break;
    case 8:
       ret = AdiReadDSInterleaveDepth(&pCommBuffer->dwValue);
       break;
    case 9:
       ret = AdiReadUSInterleaveDepth(&pCommBuffer->dwValue);
       break;
    case 10:
       ret = AdiReadInterleavedTransmittedCells(&pCommBuffer->dwValue); 
       break;
    case 11:
       ret = AdiReadFastTransmittedCells(&pCommBuffer->dwValue); 
       break;
    case 12:
       ret = AdiReadInterleavedReceivedCells(&pCommBuffer->dwValue);
       break;
    case 13:
       ret = AdiReadFastReceivedCells(&pCommBuffer->dwValue);
       break;
    case 14:
       ret = AdiReadNearEndCRC(&pCommBuffer->dwValue);
       break;
    case 15:
       ret = AdiReadFarEndCRC(&pCommBuffer->dwValue);
       break;
    case 16:
       ret = AdiReadNearEndFEC(&pCommBuffer->dwValue);
       break;
    case 17:
       ret = AdiReadFarEndFEC(&pCommBuffer->dwValue);
       break;
    case 18:
       ret = AdiReadNearEndSNRMargin(&pCommBuffer->fValue);
       f=TRUE;
       break;
    case 19:
       ret = AdiReadFarEndSNRMargin(&pCommBuffer->fValue);
       f=TRUE;
       break;
    case 20:
       ret = AdiReadFarEndAttenuation(&pCommBuffer->fValue);
       f=TRUE;   
       break;
    case 21:
       ret = AdiReadDelineation(&pCommBuffer->dwValue);
       break;
    case 22:
       ret = AdiReadATMCellsDropped_INTRL(&pCommBuffer->dwValue);
       break;
    case 23:
       ret = AdiReadATMCellsDropped_FAST(&pCommBuffer->dwValue);
       break;
    case 24:
       ret = AdiReadDSConnectionType(&pCommBuffer->dwValue);
       break;
    case 25:
       ret = AdiReadUSConnectionType(&pCommBuffer->dwValue);
       break;
    case 26:
       ret = AdiReadFramingMode(&pCommBuffer->dwValue);
       break;
    case 27:
       ret = AdiReadTrellisEncoding(&pCommBuffer->dwValue);
       break;
    case 28:
       ret = AdiReadCOVendorID(&pCommBuffer->dwValue);
       break;
    case 29:
       ret = AdiReadRTVendorID(&pCommBuffer->dwValue);
       break;
    case 30:
       printf("Please enter uiBin>");scanf("%d",&uiBin);getchar();
       ret=AdiUSBinLoading(uiBin,&NumBitPerFrame);
       if (ret==API_RETURN_SUCCESS)
         printf(" Number of Bit Per Frame: %ld\n",NumBitPerFrame);
       ResultShowed=TRUE;
       break;
    case 31:
       printf("Please enter uiBin>");scanf("%d",&uiBin);getchar();
       ret=AdiDSBinLoading(uiBin,&NumBitPerFrame,&SNR,&Gain);
       if (ret==API_RETURN_SUCCESS)
       {
         printf(" Number of Bit Per Frame: %ld\n",NumBitPerFrame);
         printf(" SNR: %f\n",SNR);  
         printf(" Gain: %f\n",Gain);  
       }
       ResultShowed=TRUE; 
       break;
    case 32:
       ret = AdiReadVPI(&pCommBuffer->dwValue);
       break;
    case 33:
       ret = AdiReadVCI(&pCommBuffer->dwValue);
       break;
    case 34:
       ret = AdiReadEncapsulation(&pCommBuffer->dwValue);
       if (ret==API_RETURN_SUCCESS)
         printf(" Encapsulation: %s\n",EncapsulationString[pCommBuffer->dwValue-1]);  
       ResultShowed=TRUE; 
       break;
    case 35:
       ret = AdiReadEncapsulation(&pCommBuffer->dwValue);
       if (ret==API_RETURN_SUCCESS)
       {
         if ((pCommBuffer->dwValue>=1) && (pCommBuffer->dwValue<=4))
           printf(" Driver Type: LAN\n");
         else
           printf(" Driver Type: WAN\n");
       }    
       ResultShowed=TRUE;
       break; 
    case 36:
       ret = AdiReadLineType(&pCommBuffer->dwValue);
       if ( (pCommBuffer->dwValue & 0x1) ==0)
         printf(" Line Type: T1.413\n");
       else
       {
          if (( pCommBuffer->dwValue & 0x18)==0) 
            printf(" LineType: G.DMT or G.Lite\n");
          else if ( (pCommBuffer->dwValue & 0x18)==1) 
            printf(" LineType: G.DMT\n");
          else if ( (pCommBuffer->dwValue & 0x18)==2) 
            printf(" LineType: G.Lite\n");
       } 
       ResultShowed=TRUE;  
       break;
    case 40:
       return;
    default:
       ret=-1;
  }
  if (ret==API_RETURN_SUCCESS)
  {
    if (!ResultShowed)
    {
      if (f)
         printf(" Value -> %f",pCommBuffer->fValue);
      else
         printf(" Value -> %s",Hex(pCommBuffer->dwValue));
    }
  }
  else
   ShowError(ret);

   printf("\n----------------------- Press Enter ----------------------------");
   do
    ch=getchar(); 
   while (ch!=10);

  }
  while (1);
}


void ApiWrite(void)
{
  BINTEST_COMMAND	binTestCmd;
  DWORD dwAdslStandard,ReturnValue,dwValue;
  UINT ret,in,binNum;
  char ch;

  do {
  ClearScr();
  printf(" 1- Modem Reset                   \n");
  printf(" 2- Write DSL Standard            \n");
  printf(" 3- Modem Test                    \n");
  printf(" 4- Put Modem in Test Signal State\n");
  printf(" 5- Put Modem in Normal State     \n");
  printf(" 6- Control Bin Test              \n");  
  printf(" 7- Read Bin Power                \n");  
  printf(" 8- Return to previous menu       \n");
  printf(" Please enter your choice>");
  scanf("%d",&in);
  getchar();

  ret=-1;
  switch (in)
  {
    /*case 1:
       ret = AdiRestartDSLConnection();
       break;*/
    case 1:
       ret = AdiModemHardReset();
       break;
    case 2:
       printf("Please enter Dsl Standard Code>");
       scanf("%ld",&dwAdslStandard);
       getchar();
       ret = AdiWriteAdslStandard(dwAdslStandard); 	
       break;
    case 3:
       ret = AdiModemSelfTest(&dwValue); 
       if (ret==API_RETURN_SUCCESS)
         switch (dwValue)
         {
            case API_MODEM_TEST_PASS:
               printf(" Modem Test Passed\n"); 
               break;
            case API_MODEM_TEST_FAILURE:
               printf(" Modem Test Failed\n");  
               break;
         }
       break;
    case 4:
       ret = AdiSetModemTestSignalState(); 
       break;
    case 5:
       ret = AdiSetModemNormalState(); 
       break;
    case 6:
       printf(" Please enter Transmit Signal Type>");
       scanf("%d",&binTestCmd.transmitSignalType);getchar();
       printf(" Please enter bin Number>");
       scanf("%d",&binTestCmd.binNumber);getchar();
       printf(" Please enter Tone Power>");
       scanf("%d",&binTestCmd.tonePower);getchar();
       printf(" Please enter Wide Band  Power>");
       scanf("%d",&binTestCmd.widebandPower);getchar();
       ret = AdiControlBinTest(&binTestCmd);
       printf(" Return Value -> %d\n",ret);
       break;
    case 7:
       printf("Please enter bin number>");
       scanf("%d",&binNum);
       getchar();
       ret = AdiReadBinPower(binNum,&ReturnValue);
       printf(" Test Return Value -> %ld\n",ReturnValue);
       break; 
    case 8:
       return;
    default:
       ret=-1;
  }
  if (ret!=API_RETURN_SUCCESS)
  {
       ShowError(ret);
  }

   printf("\n----------------------- Press Enter ----------------------------");
   do
    ch=getchar(); 
   while (ch!=10);

  }
  while (1);
}

void GeneralApiWrite(void)
{
  int ret;
  DWORD ret_val;
  UCHAR Comm[10],Offset[5],Value[5],ch;

  printf("Please enter CMV name(OPTN for example)>");
  scanf("%s",Comm);getchar();
  printf("Please enter Offset>");
  scanf("%s",Offset);getchar();
  printf("Please enter Value>");
  scanf("%s",Value);getchar();

  ret = AdiAPICmvWrite(Comm,Offset,Value,&ret_val);
  if (ret==API_RETURN_SUCCESS)
    printf(" Return Value> %s\n",Hex(ret_val));
  else
    ShowError(ret);

  printf("\n----------------------- Press Enter ----------------------------\n");
  do
   ch=getchar(); 
  while (ch!=10);
}

void GeneralApiRead(void)
{
  int ret;
  DWORD ret_val;
  UCHAR Comm[10],Offset[5],ch;

  printf("Please enter CMV name(STAT for example)>");
  scanf("%s",Comm);getchar();
  printf("Please enter Offset>");
  scanf("%s",Offset);getchar();

  ret = AdiAPICmvRead(Comm,Offset,&ret_val);
  if (ret==API_RETURN_SUCCESS)
    printf("Return Value> %s\n",Hex(ret_val));
  else
    ShowError(ret);

  printf("\n----------------------- Press Enter ----------------------------\n");
  do
   ch=getchar(); 
  while (ch!=10);
}

void DspRead(void)
{
  DWORD Address,Value;
  UCHAR ch;
  int ret;

  printf("Please enter Address>");
  scanf("%lx",&Address);getchar();
  
  ret = DSPMemoryRead(Address,&Value);
  if (ret==API_RETURN_SUCCESS)
    printf("Value: %s\n", Hex(Value));
  else
    ShowError(ret);

  printf("\n----------------------- Press Enter ----------------------------\n");
  do
   ch=getchar(); 
  while (ch!=10);
}

void DspWrite(void)
{
  DWORD Address,Value;
  UCHAR ch;
  int ret;

  printf("Please enter Address>");
  scanf("%lx",&Address);getchar();
  printf("Please enter Value>");
  scanf("%lx",&Value);getchar();
  
  ret = DSPMemoryWrite(Address,Value);

  if (ret==API_RETURN_SUCCESS)
    printf("Value is written successfully\n");
  else
    ShowError(ret);

  printf("\n----------------------- Press Enter ----------------------------\n");
  do
   ch=getchar(); 
  while (ch!=10);
}


void WriteGP(void)
{
  int ret,Value;
  char ch;

  printf("Please enter the value>");
  scanf("%d",&Value); getchar();

  ret = WriteGeneralPurposeRegister(Value);
  if (ret==API_RETURN_SUCCESS)
    printf("Value is written successfully\n");
  else
    ShowError(ret);

  printf("\n----------------------- Press Enter ----------------------------\n");
  do
   ch=getchar(); 
  while (ch!=10);
}

void ReadGP(void)
{
  int ret,Value;
  char ch;

  ret = ReadGeneralPurposeRegister(&Value);
  if (ret==API_RETURN_SUCCESS)
    printf(" Value: %x\n",Value);
  else
    ShowError(ret);

  printf("\n----------------------- Press Enter ----------------------------\n");
  do
   ch=getchar(); 
  while (ch!=10);
}

void ReadModemState(void)
{
  int ret,State;
  char ch;
  char *sState;

  ret = GetModemState(&State);
  if (ret==API_RETURN_SUCCESS)
  {
    switch (State)
    {
        case STATE_UNDEFINED:
            sState =  "Modem is unplugged from USB.\n";
            break;
        case STATE_JUST_PLUGGED_IN:
            sState = "Modem waiting for driver response.\n";
            break;
        case STATE_UNTRAIN:
        case STATE_UNTRAIN_TX:
        case STATE_UNTRAIN_RX:
        case STATE_INITIALIZING:
        case STATE_INITIALIZING_TX:
        case STATE_INITIALIZING_RX:
            sState = "Modem is initializing\n";
            break;
        case STATE_HARD_RESET_INITIATE:
        case STATE_HARD_RESET_END:
        case STATE_BOOT_WAIT:
            sState = "Modem is booting\n";
            break;
        case STATE_OPERATIONAL:
        case STATE_OPERATIONAL_TX:
        case STATE_OPERATIONAL_RX:
            sState = "Modem is operational\n";
            break;
        case STATE_TEST:
            sState = "Modem is in test state\n";
            break;
        case STATE_DISABLE:
            sState = "Modem state machine is disabled\n";
            break;
        case STATE_STALLED_FOREVER:
            sState = "Modem cannot boot. Unplug other devices and retry.\n";
            break;
        default:
            sState = "Unknown state!\n";
    }
    printf(" %s\n",sState);
  }
  else 
    ShowError(ret);
 
  printf("\n----------------------- Press Enter ----------------------------\n");
  do
   ch=getchar(); 
  while (ch!=10);
}

int main(void)
{
 char inputc;

 
 while (1)
 {
   //clear();
   ClearScr();
   printf(" 1- Detect Usb Modem\n");
   printf(" 2- Mac Address\n");
   printf(" 3- Version & Release Dates\n");
   printf(" 4- CMV Read Functions\n");
   printf(" 5- CMV Write Functions\n");
   printf(" 6- General CMV Read Functions\n");
   printf(" 7- General CMV Write Functions\n");
   printf(" 8- DSP Memory Read Function\n");
   printf(" 9- DSP Memory Write Function\n");
   //printf(" a- Unload Driver from Memory\n");
   //printf(" b- Load/Reload Driver into Memory\n");
   printf(" a- Get Modem State\n");
   printf(" b- Read General Purpose Register\n");
   printf(" c- Write General Purpose Register\n");
   printf(" d- Exit\n");
   printf(" Please enter your choice>");
   inputc= getchar();getchar();
   
   switch(inputc)
   {
     case '1':
        ApiDetect();
        break;
     case '2':
        ApiMac();
        break;
     case '3':
        ApiVersions();
        break;
     case '4':
        ApiRead();
        break;
     case '5':
        ApiWrite();
        break; 
     case '6':
        GeneralApiRead(); 
        break;
     case '7':
        GeneralApiWrite();
        break;
     case '8':
        DspRead();
        break;
     case '9':
        DspWrite();
        break;
     //case 'a':
     //   AdiDriverSoftEnumeration(0);
     //   break;
     //case 'b':
     //   AdiDriverSoftEnumeration(1);
     //   break;
     case 'a':
        ReadModemState();
        break;
     case 'b':
        ReadGP();
        break;
     case 'c':
        WriteGP();
        break;
     case 'd':
        return 0;
     default:
        break;
   }

  }
  
 return 1;
}































