/**********************************************************************************/
/*										  */
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved			  */	
/* mp_int.c									  */
/*										  */
/* This file is part of the "ADI USB ADSL Driver for Linux".			  */	
/*										  */
/* "ADI USB ADSL Driver for Linux" is free software; you can redistribute it      */
/* and/or modify it under the terms of the GNU General Public License as 	  */
/* published by the Free Software Foundation; either version 2 of the License,    */
/* or (at your option) any later version.					  */
/*										  */
/* "ADI USB ADSL Driver for Linux" is distributed in the hope that it will be     */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of	  */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		  */
/* GNU General Public License for more details.					  */
/*										  */
/* You should have received a copy of the GNU General Public License		  */
/* along with "ADI USB ADSL Driver for Linux"; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	  */
/**********************************************************************************/



/* MP interface module - mp_interface.c */



#include "sometypes.h"
#include "mp_int.h"
#include "../ADIUTIL/Cmv.h"

#define ON	1
#define OFF	0

/* local functions to support encode/decode functions */
//unsigned int 
//BOOL
bool
check_symbolic_address
(
    unsigned char *saddr
);

/* function to encode the logical data presented in the mp_message_t */
/* structure into a formatted mp message in the raw message buffer */

/* A completion code is returned by this function - should allways */
/* be OK else there's a coding problem in the calling routine */

/* local definition of the encode/decode stages */
#define	 END_ENCODE			0xffff
#define	 END_DECODE			0xffff
#define	 MP_STAGE_PREAMBLE		0x0000
#define	 MP_STAGE_FUNCTION_TYPE		0x0001
#define	 MP_STAGE_FUNCTION_SUBTYPE	0x0002
#define	 MP_STAGE_FUNCTION_SENDERID	0x0003
#define	 MP_STAGE_FUNCTION_RECEIVERID	0x0004
#define	 MP_STAGE_INDEX			0x0005
#define	 MP_STAGE_SYMBOLIC_ADDR		0x0006
#define	 MP_STAGE_OFFSET_ADDR		0x0007
#define	 MP_STAGE_DATA			0x0008

unsigned int
//BOOL
//bool
mp_msg_encode
(
    mp_message_t *msg, 
    unsigned short *mp_raw
)
{
    unsigned int ret_code = MP_ENCODE_OK;
    unsigned int mp_encode_stage;
    unsigned short temp=0;

    /* start at beginning of encoding */
    mp_encode_stage = MP_STAGE_PREAMBLE;

    /* encode the message one stage at a time */
    while (mp_encode_stage < END_ENCODE)
    {
	switch (mp_encode_stage)
	{
	case MP_STAGE_PREAMBLE:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_FUNCTION_TYPE;

	    /* check the preamble */
	    if (msg->preamble != 0x535c)
	    {
		ret_code |= MP_ENCODE_PREAMBLE_ERROR;
		ret_code |= MP_ENCODE_ERROR;
	    }

	    /* insert the preamble */
	    mp_raw[MP_RAW_PREAMBLE] = msg->preamble;
	    break;

	case MP_STAGE_FUNCTION_TYPE:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_FUNCTION_SUBTYPE;

	    /* check the function subfield */
	    switch (msg->type)
	    {
	    case MP_FUNCTION_TYPE_MEMACCESS:
	    case MP_FUNCTION_TYPE_FLASHACC:
	    case MP_FUNCTION_TYPE_MEDIRECTIVE:
		/* legal function types - add into field */
		temp = MP_FUNCTION_TYPE_MASK & 
		    (msg->type & 0x000f) << MP_FUNCTION_TYPE_LSB;
		break;

	    default:
		/* illegal function type - set return code */
		ret_code |= MP_ENCODE_TYPE_ERROR;
		ret_code |= MP_ENCODE_ERROR;
		break;
	    }
	    break;

	case MP_STAGE_FUNCTION_SUBTYPE:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_FUNCTION_SENDERID;

	    /* check that the function type was correct, if it */
	    /* wasn't, we can't check the sub-type field */
	    if ((ret_code & MP_ENCODE_TYPE_ERROR) != 0)
	    {
		/* type field was in error, don't check sub-type */
	    }
	    else
	    {
		switch (msg->type)
		{
		case MP_FUNCTION_TYPE_MEMACCESS:
		    /* check the function subfield */
		    switch (msg->sub_type)
		    {
		    case SUBTYPE_MEMACCESS_REQREAD:
		    case SUBTYPE_MEMACCESS_REQWRITE:
			/* legal function subtypes - and into field */
			temp |= (MP_FUNCTION_SUBTYPE_MASK & 
			    ((msg->sub_type & 0x000f) << 
			    MP_FUNCTION_SUBTYPE_LSB));
			break;

		    default:
			/* illegal function type - set return code */
			ret_code |= MP_ENCODE_SUBTYPE_ERROR;
			ret_code |= MP_ENCODE_ERROR;
			break;
		    }
		    break;

		case MP_FUNCTION_TYPE_FLASHACC:
		    /* check the function subfield */
		    switch (msg->sub_type)
		    {
		    case SUBTYPE_FLASHACC_ENKERNELREBOOTREQ:
		    case SUBTYPE_FLASHACC_ENMODEMREBOOTREQ:
		    case SUBTYPE_FLASHACC_DECOMPMODEMAPPLREQ:
		    case SUBTYPE_FLASHACC_STREAMEDDATAWRREQ:
		    case SUBTYPE_FLASHACC_STREAMEDDATAWRBLOCK:
		    case SUBTYPE_FLASHACC_READREQ:
		    case SUBTYPE_FLASHACC_CALCMODEMCRCREQ:
			/* legal function subtypes - and into field */
			temp |= (MP_FUNCTION_SUBTYPE_MASK & 
			    ((msg->sub_type & 0x000f) << 
			    MP_FUNCTION_SUBTYPE_LSB));
			break;

		    default:
			/* illegal function type - set return code */
			ret_code |= MP_ENCODE_SUBTYPE_ERROR;
			ret_code |= MP_ENCODE_ERROR;
			break;
		    }
		    break;

		case MP_FUNCTION_TYPE_MEDIRECTIVE:
		    /* check the function subfield */
		    switch (msg->sub_type)
		    {
		    case SUBTYPE_MEDIRECTIVE_REBOOT:
			/* legal function subtypes - and into field */
			temp |= (MP_FUNCTION_SUBTYPE_MASK & 
			    ((msg->sub_type & 0x000f) << 
			    MP_FUNCTION_SUBTYPE_LSB));
			break;

		    default:
			/* illegal function type - set return code */
			ret_code |= MP_ENCODE_SUBTYPE_ERROR;
			ret_code |= MP_ENCODE_ERROR;
			break;
		    }
		    break;

		default:
		    /* type is invalid and passed last test - can't happen */
		    /* got a serious error here */
		    //error("SW Internal ERROR 'msg->type'");
		    break;
		}		/* end switch msg->type */
	    }			/* end else */
	    break;

	case MP_STAGE_FUNCTION_SENDERID:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_FUNCTION_RECEIVERID;

	    /* check the function subfield */
	    switch (msg->sender_id)
	    {
	    case SUBTYPE_ID_ME:
		/* legal function sender_id - and into field */
		temp |= MP_FUNCTION_SENDERID_MASK & ((msg->sender_id & 0x000f) << MP_FUNCTION_SENDERID_LSB);
		break;

	    default:
		/* illegal function type - set return code */
		ret_code |= MP_ENCODE_SENDERID_ERROR;
		ret_code |= MP_ENCODE_ERROR;
		break;
	    }
	    break;

	case MP_STAGE_FUNCTION_RECEIVERID:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_INDEX;

	    /* check the function subfield */
	    switch (msg->receiver_id)
	    {
	    case SUBTYPE_ID_ADSL:
		/* legal function receiver_id - and into field */
		temp |= MP_FUNCTION_RECEIVER_MASK & 
		    ((msg->receiver_id & 0x000f) << MP_FUNCTION_RECEIVER_LSB);

		/* write the function word to the message */
		mp_raw[MP_RAW_FUNCTION] = temp;
		break;

	    default:
		/* illegal function type - set return code */
		ret_code = MP_ENCODE_ERROR;
		/* stop processing the message */
		mp_encode_stage = END_ENCODE;
		break;
	    }
	    break;

	case MP_STAGE_INDEX:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_SYMBOLIC_ADDR;

	    /* insert the index */
	    mp_raw[MP_RAW_INDEX] = msg->index;
	    break;


	case MP_STAGE_SYMBOLIC_ADDR:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_OFFSET_ADDR;

	    if (check_symbolic_address(msg->symb_addr) == true)
	    {
		/* symbolic address is valid, enter into msg */

		/*
		 * pack the two most significant chars into 
		 * the mp symb addr field 
		 */
		temp = (msg->symb_addr[0] & 0x00ff);
		temp <<= 8;
		temp |= (msg->symb_addr[1] & 0x00ff);
		mp_raw[MP_RAW_SYMB_ADDR_HIGH] = temp;

		/*
		 * pack the two least significant chars into 
		 * the mp symb addr field 
		 */
		temp = (msg->symb_addr[2] & 0x00ff);
		temp <<= 8;
		temp |= (msg->symb_addr[3] & 0x00ff);
		mp_raw[MP_RAW_SYMB_ADDR_LOW] = temp;
	    }
	    else
	    {
		/* symbolic address invalid - set return code */
		ret_code |= MP_ENCODE_SYMB_ADDR;
		ret_code |= MP_ENCODE_ERROR;
	    }
	    break;

	case MP_STAGE_OFFSET_ADDR:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = MP_STAGE_DATA;

	    /* not going to check this - just insert regardless */
	    mp_raw[5] = msg->offset_addr;
	    break;

	case MP_STAGE_DATA:
	    /* set mp_encode_stage to process next word */
	    mp_encode_stage = END_ENCODE;

	    /* can't check this - just insert regardless */
	    /* MSW */
	    mp_raw[6] = (unsigned short) ((msg->data & 0xffff0000) >> 16);
	    /* LSW */
	    mp_raw[7] = (unsigned short) (msg->data & 0x0000ffff);
	    break;
	}
    };

    /* all done */
    return (ret_code);
}

/* function to decode the raw mp message presented in the raw */
/* mp_raw array into the logical data of the mp_message_t structure */

/* A completion code is returned by this function, should allways be */
/* OK unless there is a hw, comms or ADSL modem firmware problem */
unsigned int
// BOOL
//bool
mp_msg_decode
(
    mp_message_t * msg, 
    unsigned short *mp_raw
)
{
    unsigned int ret_code = MP_DECODE_OK;
    unsigned int mp_decode_stage;
    unsigned long int temp32;

    /* start at beginning of encoding */
    mp_decode_stage = MP_STAGE_PREAMBLE;

    /* encode the message one stage at a time */
    while (mp_decode_stage < END_DECODE)
    {
	switch (mp_decode_stage)
	{
	case MP_STAGE_PREAMBLE:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_FUNCTION_TYPE;

	    /* extract the preamble */
	    msg->preamble = mp_raw[MP_RAW_PREAMBLE];

	    /* check the preamble */
	    if (msg->preamble != 0x535c)
	    {
		ret_code |= MP_DECODE_PREAMBLE_ERROR;
		ret_code |= MP_DECODE_ERROR;
	    }
	    break;

	case MP_STAGE_FUNCTION_TYPE:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_FUNCTION_SUBTYPE;

	    /* extract the type field */
	    msg->type = ((MP_FUNCTION_TYPE_MASK & mp_raw[MP_RAW_FUNCTION]) 
		>> MP_FUNCTION_TYPE_LSB);

	    /* check the function subfield */
	    switch (msg->type)
	    {
	    case MP_FUNCTION_TYPE_MEMACCESS:
	    case MP_FUNCTION_TYPE_FLASHACC:
	    case MP_FUNCTION_TYPE_ADSLDIRECTIVE:
		/* legal function types  */
		break;

	    default:
		/* illegal function type - set return code */
		ret_code |= MP_DECODE_TYPE_ERROR;
		ret_code |= MP_DECODE_ERROR;
		break;
	    }
	    break;

	case MP_STAGE_FUNCTION_SUBTYPE :
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_FUNCTION_SENDERID;

	    /* extract the sub_type field */
	    msg->sub_type = ((MP_FUNCTION_SUBTYPE_MASK & mp_raw[MP_RAW_FUNCTION]) >> MP_FUNCTION_SUBTYPE_LSB);

	    /* check that the function type was correct, if it */
	    /* wasn't, we can't check the sub-type field */
	    if ((ret_code & MP_DECODE_TYPE_ERROR) != 0)
	    {
		/* type field was in error, don't check sub-type */
	    }
	    else
	    {
		switch (msg->type)
		{
		case MP_FUNCTION_TYPE_MEMACCESS:
		    /* check the function subfield */
		    switch (msg->sub_type)
		    {
		    case SUBTYPE_MEMACCESS_REPLYREAD:
		    case SUBTYPE_MEMACCESS_REPLYWRITE:
			/* legal function subtypes - and into field */
			break;

		    default:
			/* illegal function type - set return code */
			ret_code |= MP_DECODE_SUBTYPE_ERROR;
			ret_code |= MP_DECODE_ERROR;
			break;
		    }
		    break;

		case MP_FUNCTION_TYPE_FLASHACC:
		    /* check the function subfield */
		    switch (msg->sub_type)
		    {
		    case SUBTYPE_FLASHACC_ENKERNELREBOOTREPLY:
		    case SUBTYPE_FLASHACC_ENMODEMREBOOTREPLY:
		    case SUBTYPE_FLASHACC_DECOMPMODEMAPPLREPLY:
		    case SUBTYPE_FLASHACC_STREAMEDDATAWRREPLY:
		    case SUBTYPE_FLASHACC_STREAMEDDATAWRBLOCKREPLY:
		    case SUBTYPE_FLASHACC_READREPLY:
			/* legal function subtypes - and into field */
			break;

		    default:
			/* illegal function type - set return code */
			ret_code |= MP_DECODE_SUBTYPE_ERROR;
			ret_code |= MP_DECODE_ERROR;
			break;
		    }
		    break;

		case MP_FUNCTION_TYPE_FLASHACCERR:
		    /* check the function subfield */
		    switch (msg->sub_type)
		    {
		    case SUBTYPE_FLASHACCERR_DECOMPRESSERR:
		    case SUBTYPE_FLASHACCERR_READCOMPERR:
			/* legal function subtypes - and into field */
			break;

		    default:
			/* illegal function type - set return code */
			ret_code |= MP_DECODE_SUBTYPE_ERROR;
			ret_code |= MP_DECODE_ERROR;
			break;
		    }
		    break;

		case MP_FUNCTION_TYPE_ADSLDIRECTIVE:
		    /* check the function subfield */
		    switch (msg->sub_type)
		    {
		    case SUBTYPE_ADSLDIRECTIVE_KERNELREADY:
		    case SUBTYPE_ADSLDIRECTIVE_MODEMREADY:
		    case SUBTYPE_ADSLDIRECTIVE_MODEMCRCERROR:
			/* legal function subtypes - and into field */
			break;

		    default:
			/* illegal function type - set return code */
			ret_code |= MP_DECODE_SUBTYPE_ERROR;
			ret_code |= MP_DECODE_ERROR;
			break;
		    }
		    break;

		default:
		    /* type is invalid and passed last test - can't happen */
		    /* got a serious error here */
		    //error("SW Internal ERROR 'msg->type'");
		    break;
		}		/* end switch msg->type */
	    }			/* end else */
	    break;

	case MP_STAGE_FUNCTION_SENDERID:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_FUNCTION_RECEIVERID;

	    /* extract the sender_id */
	    msg->sender_id = ((MP_FUNCTION_SENDERID_MASK & mp_raw[MP_RAW_FUNCTION]) >> MP_FUNCTION_SENDERID_LSB);

	    /* check the function subfield */
	    switch (msg->sender_id)
	    {
	    case SUBTYPE_ID_ADSL:
		/* legal function sender_id - and into field */
		break;

	    default:
		/* illegal function type - set return code */
		ret_code |= MP_DECODE_SENDERID_ERROR;
		ret_code |= MP_DECODE_ERROR;
		break;
	    }
	    break;

	case MP_STAGE_FUNCTION_RECEIVERID:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_INDEX;

	    /* extract the sender_id */
	    msg->receiver_id = ((MP_FUNCTION_RECEIVER_MASK & mp_raw[MP_RAW_FUNCTION]) >> MP_FUNCTION_RECEIVER_LSB);

	    /* check the function subfield */
	    switch (msg->receiver_id)
	    {
	    case SUBTYPE_ID_ME:
		/* legal function receiver_id - and into field */
		break;

	    default:
		/* illegal function type - set return code */
		ret_code = MP_DECODE_ERROR;
		/* stop processing the message */
		mp_decode_stage = END_DECODE;
		break;
	    }
	    break;

	case MP_STAGE_INDEX:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_SYMBOLIC_ADDR;

	    /* insert the index */
	    msg->index = mp_raw[MP_RAW_INDEX];
	    break;


	case MP_STAGE_SYMBOLIC_ADDR:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_OFFSET_ADDR;

	    if (msg->type != MP_FUNCTION_TYPE_ADSLDIRECTIVE)
	    {
		/* extract the symbolic address */
		msg->symb_addr[0] = (unsigned char) ((unsigned short) (mp_raw[MP_RAW_SYMB_ADDR_HIGH] & 0xff00) >> 8);
		msg->symb_addr[1] = (unsigned char) ((unsigned short) (mp_raw[MP_RAW_SYMB_ADDR_HIGH] & 0x00ff));
		msg->symb_addr[2] = (unsigned char) ((unsigned short) (mp_raw[MP_RAW_SYMB_ADDR_LOW] & 0xff00) >> 8);
		msg->symb_addr[3] = (unsigned char) ((unsigned short) (mp_raw[MP_RAW_SYMB_ADDR_LOW] & 0x00ff));

		if (check_symbolic_address(msg->symb_addr) == true)
		{
		    /* symbolic address is valid */
		}
		else
		{
		    /* symbolic address invalid - set return code */
		    ret_code |= MP_DECODE_SYMB_ADDR;
		    ret_code |= MP_DECODE_ERROR;
		}
	    }
	    else
	    {
		/* can't check the symbolic address for ADSL */
		/* directive messages */
	    }
	    break;

	case MP_STAGE_OFFSET_ADDR:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = MP_STAGE_DATA;

	    /* not going to check this - just insert regardless */
	    msg->offset_addr = mp_raw[MP_RAW_OFFSET_ADDR];
	    break;

	case MP_STAGE_DATA:
	    /* set mp_decode_stage to process next word */
	    mp_decode_stage = END_DECODE;

	    /* can't check this - just insert regardless */
	    temp32 = (unsigned long int) ((unsigned long int) (mp_raw[MP_RAW_DATA_HIGH]) & 0x0000ffff);
	    temp32 <<= 16;
	    temp32 |= (unsigned long int) ((unsigned long int) (mp_raw[MP_RAW_DATA_LOW]) & 0x0000ffff);
	    msg->data = temp32;
	    break;
	}
    };

    /* all done */
    return (ret_code);
}

/* function to check that a symbolic address is valid */
/* there is probably a much better/quicker way of doing */
/* this - but too little time for such niceties ... */

/* local definition of symbolic address */
#define		NUM_CMVS	20

static char symb_addresses[NUM_CMVS][4] =
{
    {'A', 'D', 'P', 'T'},
    {'C', 'N', 'T', 'L'},
    {'C', 'O', 'D', 'E'},
    {'D', 'I', 'A', 'G'},
    {'D', 'O', 'P', 'T'},
    {'F', 'L', 'A', 'G'},
    {'I', 'N', 'F', 'O'},
    {'I', 'N', 'T', 'L'},
    {'O', 'P', 'T', 'N'},
    {'P', 'F', 'C', 'L'},
    {'P', 'F', 'R', 'X'},
    {'P', 'F', 'T', 'X'},
    {'P', 'S', 'D', 'M'},
    {'R', 'A', 'T', 'E'},
    {'R', 'X', 'D', 'A'},
    {'S', 'T', 'A', 'T'},
    {'T', 'E', 'S', 'T'},
    {'T', 'O', 'N', 'E'},
    {'T', 'X', 'D', 'A'},
    {'U', 'O', 'P', 'T'},
};

#define	CODE_SEARCHING	0x0
#define	CODE_PASSED		0x1
#define	CODE_FAILED		0x2

//unsigned int 
//BOOL
bool
check_symbolic_address(unsigned char *saddr)
{
    bool ret_code = false;
    unsigned int search_code = CODE_SEARCHING;
    unsigned int cmv = 0;
    unsigned int letter;

    /* search the reference array for a match */
    while ((cmv < NUM_CMVS) && (search_code == CODE_SEARCHING))
    {
	/* try to match the CMV to the current reference array CMV */
	letter = 0;
	while (letter < 4 && (saddr[letter] == symb_addresses[cmv][letter]))
	{
	    letter++;
	};

	switch (letter)
	{
	case 0:
	    /* no match on first char - check that first char of CMV being */
	    /* checked is not before the first char of this entry in */
	    /* the alphabet, if it is we can stop looking as entries in the */
	    /* reference array are in ascending order */
	    if (saddr[0] < symb_addresses[cmv][0])
	    {
		/* give up looking, already too far into alphabet */
		search_code = CODE_FAILED;
		ret_code = false;
	    }
	    else
	    {
		/* not too far yet, keep looking */
	    }
	    break;

	case 4:
	    /* found a match */
	    search_code = CODE_PASSED;
	    ret_code = true;
	    break;

	default:
	    /* partial match, keep looking */
	    break;
	}
	cmv++;
    }

    return TRUE; 
	// return ret_code;
}


