/**********************************************************************************/
/*										  */
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved			  */	
/* mp_int.h 									  */
/*										  */
/* This file is part of the "ADI USB ADSL Driver for Linux".			  */	
/*										  */
/* "ADI USB ADSL Driver for Linux" is free software; you can redistribute it      */
/* and/or modify it under the terms of the GNU General Public License as 	  */
/* published by the Free Software Foundation; either version 2 of the License,    */
/* or (at your option) any later version.					  */
/*										  */
/* "ADI USB ADSL Driver for Linux" is distributed in the hope that it will be     */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of	  */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		  */
/* GNU General Public License for more details.					  */
/*										  */
/* You should have received a copy of the GNU General Public License		  */
/* along with "ADI USB ADSL Driver for Linux"; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	  */
/**********************************************************************************/

#ifndef _MP_INTERFACE
#define _MP_INTERFACE 

/* MP logical structure for encode source, decode destination */
struct mp_message
{
    /* message function field */
    unsigned short preamble;
    unsigned short type;
    unsigned short sub_type;
    unsigned short sender_id;
    unsigned short receiver_id;

    /* message index field */
    unsigned short index;

    /* message symbolic address and offset fields */
    unsigned char symb_addr[4];
    unsigned short offset_addr;

    /* message data field */
    unsigned long data;
};

typedef struct mp_message mp_message_t;

/* MP raw buffer offset definitions - zero based */
#define MP_RAW_PREAMBLE		0x0
#define MP_RAW_FUNCTION		0x1
#define MP_RAW_INDEX		0x2
#define MP_RAW_SYMB_ADDR_HIGH	0x3
#define MP_RAW_SYMB_ADDR_LOW	0x4
#define MP_RAW_OFFSET_ADDR	0x5
#define MP_RAW_DATA_HIGH	0x6
#define MP_RAW_DATA_LOW		0x7

/* function prototypes */
unsigned int 
//BOOL
//bool
mp_msg_encode
(
    mp_message_t *msg, 
    unsigned short mp_raw[8]
);

unsigned int 
//BOOL
//bool
mp_msg_decode
(
    mp_message_t *msg, 
    unsigned short mp_raw[8]
);

/* mp_msg_encode/decode function return values */
#define	MP_ENCODE_OK			0x00000000
#define	MP_ENCODE_ERROR			0x00000001
#define	MP_ENCODE_PREAMBLE_ERROR	0x00000002
#define	MP_ENCODE_TYPE_ERROR		0x00000004
#define	MP_ENCODE_SUBTYPE_ERROR		0x00000008
#define	MP_ENCODE_SENDERID_ERROR	0x00000010
#define	MP_ENCODE_RECEIVERID_ERROR	0x00000020
#define	MP_ENCODE_SYMB_ADDR		0x00000040

#define MP_DECODE_OK			0x00000000
#define	MP_DECODE_ERROR			0x00000001
#define	MP_DECODE_PREAMBLE_ERROR	0x00000002
#define	MP_DECODE_TYPE_ERROR		0x00000004
#define	MP_DECODE_SUBTYPE_ERROR		0x00000008
#define	MP_DECODE_SENDERID_ERROR	0x00000010
#define	MP_DECODE_RECEIVERID_ERROR	0x00000020
#define	MP_DECODE_SYMB_ADDR		0x00000040

/* MP command/response block bit definitions & masks */

/* function field, type subfield */

#define MP_FUNCTION_TYPE_MEMACCESS	0x1
#define MP_FUNCTION_TYPE_MSGDECERR	0x2
#define MP_FUNCTION_TYPE_MSGACCERR	0x3
#define MP_FUNCTION_TYPE_FLASHACC	0x4
#define MP_FUNCTION_TYPE_FLASHACCERR	0x5
#define MP_FUNCTION_TYPE_MEDIRECTIVE	0x6
#define MP_FUNCTION_TYPE_ADSLDIRECTIVE	0x7

/* function field, subtype subfield */

#define	SUBTYPE_MEMACCESS_REQREAD		0x0
#define	SUBTYPE_MEMACCESS_REQWRITE		0x1
#define	SUBTYPE_MEMACCESS_REPLYREAD		0x2
#define	SUBTYPE_MEMACCESS_REPLYWRITE		0x3

#define	SUBTYPE_MEMACCESSERROR_UNKNOWNADDR	0x1
#define	SUBTYPE_MEMACCESSERROR_DATANOTRDY	0x2
#define	SUBTYPE_MEMACCESSERROR_ILLEGALREAD	0x3
#define	SUBTYPE_MEMACCESSERROR_ILLEGALWRITE	0x4
#define	SUBTYPE_MEMACCESSERROR_DATANOTAVAIL	0x5

#define	SUBTYPE_MSGDECERR_LENGTHERROR		0x1
#define	SUBTYPE_MSGDECERR_PREAMBLEERROR		0x2
#define	SUBTYPE_MSGDECERR_UNKNOWNTYPE		0x3
#define	SUBTYPE_MSGDECERR_CRCERROR		0x4

#define SUBTYPE_FLASHACC_ENKERNELREBOOTREQ	0x0
#define SUBTYPE_FLASHACC_ENMODEMREBOOTREQ	0x1
#define SUBTYPE_FLASHACC_DECOMPMODEMAPPLREQ	0x2
#define SUBTYPE_FLASHACC_STREAMEDDATAWRREQ	0x3
#define SUBTYPE_FLASHACC_STREAMEDDATAWRBLOCK	0x4
#define SUBTYPE_FLASHACC_READREQ		0x5
#define SUBTYPE_FLASHACC_CALCMODEMCRCREQ	0x7
#define SUBTYPE_FLASHACC_ENKERNELREBOOTREPLY	0x8
#define SUBTYPE_FLASHACC_ENMODEMREBOOTREPLY	0x9
#define SUBTYPE_FLASHACC_DECOMPMODEMAPPLREPLY	0xa
#define SUBTYPE_FLASHACC_STREAMEDDATAWRREPLY	0xb
#define SUBTYPE_FLASHACC_STREAMEDDATAWRBLOCKREPLY	0xc
#define SUBTYPE_FLASHACC_READREPLY			0xd
#define SUBTYPE_FLASHACC_CALCMODEMCRCREPLY		0xf

#define SUBTYPE_FLASHACCERR_READCOMPERR			0x0
#define SUBTYPE_FLASHACCERR_DECOMPRESSERR		0x1

#define	SUBTYPE_MEDIRECTIVE_REBOOT			0x0

#define	SUBTYPE_ADSLDIRECTIVE_KERNELREADY		0x0
#define	SUBTYPE_ADSLDIRECTIVE_MODEMREADY		0x1
#define	SUBTYPE_ADSLDIRECTIVE_MODEMCRCERROR		0x2

/* function field, sender/receiver_id subfield */

#define	SUBTYPE_ID_ADSL					0x0
#define	SUBTYPE_ID_ME					0x1

/* bit positions, masks and shift counts */

#define	MP_FUNCTION_TYPE_MASK			0xf000
#define	MP_FUNCTION_SUBTYPE_MASK		0x0f00
#define	MP_FUNCTION_SENDERID_MASK		0x00f0
#define	MP_FUNCTION_RECEIVER_MASK		0x000f

#define	MP_FUNCTION_TYPE_LSB			0xc
#define	MP_FUNCTION_SUBTYPE_LSB			0x8
#define	MP_FUNCTION_SENDERID_LSB		0x4
#define	MP_FUNCTION_RECEIVER_LSB		0x0

#endif //_MP_INTERFACE
