/**********************************************************************************/
/*										  */
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved			  */	
/* othapi.c 									  */
/*										  */
/* This file is part of the "ADI USB ADSL Driver for Linux".			  */	
/*										  */
/* "ADI USB ADSL Driver for Linux" is free software; you can redistribute it      */
/* and/or modify it under the terms of the GNU General Public License as 	  */
/* published by the Free Software Foundation; either version 2 of the License,    */
/* or (at your option) any later version.					  */
/*										  */
/* "ADI USB ADSL Driver for Linux" is distributed in the hope that it will be     */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of	  */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		  */
/* GNU General Public License for more details.					  */
/*										  */
/* You should have received a copy of the GNU General Public License		  */
/* along with "ADI USB ADSL Driver for Linux"; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	  */
/**********************************************************************************/


#include "linuxusbapi.h"
int delete_module(const char *name);

UINT	uiAPIDetectedDevice = API_DEVICE_USB;
UINT	uiAPILastInternalError = INTR_SUCCESS;

UINT IOReady(void)
{
 int ret,init=0;
 ret =  AdiIOCTLCommand(ADIUSBADSLMDRAEDY,&init);
 if ( (ret==-1) || (init!=1) )
   return API_RETURN_COMM_ERROR;
 else
   return API_RETURN_SUCCESS;
}

BOOLEAN IsUSBModemConnected(void)
{

  int ret,State;
  if (IOReady()!=API_RETURN_SUCCESS)
    return FALSE;
  //sleep(1); 
  ret = GetModemState(&State);  
  if (ret!=API_RETURN_SUCCESS)
    return FALSE;
  if ( (State == STATE_JUST_PLUGGED_IN) || (State == STATE_UNDEFINED) )
    return FALSE;
  return TRUE;
}

UINT AdiDetectDevice(void)
{
 
 if ( IsUSBModemConnected() )
   return API_DEVICE_USB;

 return API_DEVICE_NOTDETECTED;
}


UINT AdiAPIMacAddress(UCHAR *u64MacAddress, UCHAR *sMac)
{
 int ret_val;
 int cntr = 0;
 UInt8       MAC[UNIQUE_ID_SIZE];

 memset(MAC,0,sizeof(UInt8)*UNIQUE_ID_SIZE);

 ret_val = AdiIOCTLCommand(ADIUSBADSLMAC,&MAC);
 if ( ret_val < 0 )
 {
    u64MacAddress = API_DEFAULT_RETURN_VALUE;
    uiAPILastInternalError = INTR_DRIVER_ERROR;
    return API_RETURN_COMM_ERROR;
 }
 else
 {	
    char str[2];
       
    for ( cntr = 0; cntr < UNIQUE_ID_SIZE; cntr++)
    {
       u64MacAddress[cntr] = (UCHAR)MAC[cntr];
       sprintf(str,"%2.2x",u64MacAddress[cntr]);
       sMac[2*cntr] = str[0];
       sMac[2*cntr+1] = str[1]; 
    }
     
    sMac[UNIQUE_ID_SIZE*2] = 0;
    return API_RETURN_SUCCESS;
 }
}



UCHAR   firm_ver[MAX_VER_LEN];
UCHAR   dsp_ver [MAX_VER_LEN];
UCHAR   drv_ver [MAX_VER_LEN];

UINT    GetDriverVersion(UCHAR *sVer)
{

 int ret_val;

 ret_val = AdiIOCTLCommand(ADIUSBADSLVER,sVer); 
 return ret_val;
}

CString	AdiAPIGetVersion(int SoftwareCode)
{
  CString sVersion;
  DWORD dwVersion;


  switch (SoftwareCode)
  {
    case API_DRIVER_VERSION:
      if (GetDriverVersion(drv_ver)>0)
        sVersion = drv_ver; 
      else
        sVersion = "Driver Doesn't Respond";
      break;

    case API_LIB_RELEASE_VERSION:
      sVersion = ApiVersion;
      break;

    case API_FIRMWARE_VERSION: // should be changed to firmware (instead of DSP!)
      if ( AdiReadDspVersion(&dwVersion) == API_RETURN_SUCCESS )
	sprintf(firm_ver,"%lx",dwVersion);
      sVersion = firm_ver;
      break;

    case API_CMV_DSP_VERSION:
      if ( AdiReadDspVersion(&dwVersion) == API_RETURN_SUCCESS )
	sprintf(dsp_ver,"%lx",dwVersion);
      sVersion = dsp_ver;
      break;


    default:
	sVersion = "SW Code Invalid!";
	break;
  }
	
  return sVersion;
}


CString AdiAPILibReleaseDate(void)
{
	CString sReleaseDate;

	sReleaseDate = ApiReleaseDate;
	return sReleaseDate;
}


UINT DSPMemoryRead (DWORD dwAddress, DWORD *pReturnValue)
{
 UserDspRWInf DspRWInfo;
 UINT ret;
 int ret_val;
  
 DspRWInfo.Addr = dwAddress;
 DspRWInfo.ReturnValue=FALSE;

 ret_val = AdiIOCTLCommand(ADIUSBADSLDSPMEMR,&DspRWInfo); 

 
 if (ret_val<0)
   return API_RETURN_COMM_ERROR;

 if (DspRWInfo.ReturnValue==TRUE)
 {
   *pReturnValue = DspRWInfo.Value; 
   ret = API_RETURN_SUCCESS;
 }
 else
   ret = API_RETURN_HARDWARE_ERROR;

 return ret;
}

UINT DSPMemoryWrite (DWORD dwAddress, DWORD Value)
{
 UserDspRWInf DspRWInfo;
 UINT ret;
 int ret_val;
  
 DspRWInfo.Addr = dwAddress;
 DspRWInfo.Value = Value;

 ret_val = AdiIOCTLCommand(ADIUSBADSLDSPMEMW,&DspRWInfo); 

 if (ret_val<0)
   return API_RETURN_COMM_ERROR;

 if (DspRWInfo.ReturnValue==TRUE)
 {
   ret = API_RETURN_SUCCESS;
 }
 else
   ret = API_RETURN_HARDWARE_ERROR;

 return ret;
}

UINT	AdiDriverSoftEnumeration(int Load)
{
 FILE *fp;
 pid_t pid;
 uid_t uid;


 uid = getuid();
 setuid(0);
 if (Load==0)  // Unload Driver
   delete_module("adiusbadsl");
 else         // Reload Driver
 {
   pid = fork();
   if (pid!=0) 
   {
     if (!IsUSBModemConnected()) return 0;
     sleep(2);
     pid = fork();
     if (pid!=0) {setuid(uid);return 0;}
     execlp("adictrl","",NULL);  
   }
   else
   {
     delete_module("adiusbadsl");
     fp = fopen("/tmp/insmod","w+");
     if (fp==NULL) {setuid(uid);return -1;}
     fprintf(fp,"/sbin/insmod adiusbadsl > NULL \r\n rm /tmp/insmod ");
     fclose(fp);
     chmod("/tmp/insmod",S_IXUSR|S_IXGRP|S_IXOTH);
     execlp("/tmp/insmod","",NULL); 
   }
 }
 setuid(uid);
 return 0;
}

UINT	ReadGeneralPurposeRegister(UINT *pReturnValue)
{
   UINT ret;
   
   ret =  AdiIOCTLCommand(ADIUSBADSLREADGP,pReturnValue);
   if (ret==-1)
     return API_RETURN_COMM_ERROR;
   else
     return API_RETURN_SUCCESS;
}

UINT	WriteGeneralPurposeRegister(UINT Value)
{
   UINT ret;
   
   ret =  AdiIOCTLCommand(ADIUSBADSLWRITEGP,&Value);
   if (ret==-1)
     return API_RETURN_COMM_ERROR;
   else
     return API_RETURN_SUCCESS;

}

UINT  SetModemState(UINT State)
{
   UINT ret;
   
   ret =  AdiIOCTLCommand(ADIUSBADSLSTATE,&State);
   if (ret==-1)
     return API_RETURN_COMM_ERROR;
   else
     return API_RETURN_SUCCESS;

}


UINT  GetModemState(UINT *State)
{
   UINT ret;

   ret =  AdiIOCTLCommand(ADIUSBADSLGETSTATE,State);
   if (ret==-1)
     return API_RETURN_COMM_ERROR;
   else
     return API_RETURN_SUCCESS;

}


int ReadParams(ParamReadType *p)
{
  int ret;
  ret =  AdiIOCTLCommand(ADIUSBADSLREADPAR,p);
   if (ret==-1)
     return API_RETURN_COMM_ERROR;
   else
     return API_RETURN_SUCCESS;
}

UINT AdiReadVPI(DWORD *dwValue)
{
  int ret;
  ParamReadType p;
  ret =  ReadParams(&p);
  (*dwValue) = p.VPI;
  if (ret==-1)
    return API_RETURN_COMM_ERROR;
  else
    return API_RETURN_SUCCESS;
}

UINT AdiReadVCI(DWORD *dwValue)
{
  int ret;
  ParamReadType p;
  ret =  ReadParams(&p);
  (*dwValue) = p.VCI;
  if (ret==-1)
    return API_RETURN_COMM_ERROR;
  else
    return API_RETURN_SUCCESS;
}

UINT AdiReadEncapsulation(DWORD *dwValue)
{
  int ret;
  ParamReadType p;
  ret =  ReadParams(&p);
  (*dwValue) = p.Encapsulation;
  if (ret==-1)
    return API_RETURN_COMM_ERROR;
  else
    return API_RETURN_SUCCESS;
}







